<?php
/**
 * The Auth_gollem:: class provides an Gollem implementation of the Horde
 * authentication system.
 *
 * Required parameters:
 * ====================
 * NONE
 *
 * Optional parameters:
 * ====================
 * NONE
 *
 * 
 * $Horde: gollem/lib/Auth/gollem.php,v 1.4 2005/01/03 14:34:54 jan Exp $
 *
 * Copyright 2004-2005 Michael Slusarz <slusarz@curecanti.org>
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 *
 * @author  Michael Slusarz <slusarz@curecanti.org>
 * @version $Revision: 1.4 $
 * @since   Horde 3.0
 * @package Horde_Auth
 */
class Auth_gollem extends Auth {

    /**
     * Constructs a new Gollem authentication object.
     *
     * @access public
     *
     * @param optional array $params  A hash containing connection parameters.
     */
    function Auth_gollem($params = array())
    {
    }

    /**
     * Find out if a set of login credentials are valid, and if
     * requested, mark the user as logged in in the current session.
     *
     * @access public
     *
     * @param optional string $userID      The userID to check.
     * @param optional array $credentials  The credentials to check.
     * @param optional boolean $login      Whether to log the user in. If
     *                                     false, we'll only test the
     *                                     credentials and won't modify
     *                                     the current session.
     *
     * @return boolean  Whether or not the credentials are valid.
     */
    function authenticate($userID = null, $credentials = array(),
                          $login = false)
    {
        // Check for for hordeauth.
        if (empty($_SESSION['gollem']['backend_key'])) {
            if (Gollem::canAutoLogin()) {
                $backend_key = Gollem::getPreferredBackend();

                require GOLLEM_BASE . '/config/backends.php';
                $ptr = &$backends[$backend_key];
                if (!empty($ptr['hordeauth'])) {
                    if (strcasecmp($ptr['hordeauth'], 'full') == 0) {
                        $user = Auth::getAuth();
                    } else {
                        $user = Auth::getBareAuth();
                    }
                    $pass = Auth::getCredential('password');

                    require_once GOLLEM_BASE . '/lib/Session.php';

                    if (Gollem_Session::createSession($backend_key, $user, $pass)) {
                        $entry = sprintf('Login success for %s [%s] to {%s}',
                                         $user, $_SERVER['REMOTE_ADDR'],
                                         $backend_key);
                        Horde::logMessage($entry, __FILE__, __LINE__, 
                                          PEAR_LOG_NOTICE);
                        return true;
                    }
                }
            }
        }

        if (empty($userID) && 
            !empty($_SESSION['gollem']['params']['username'])) {
            $userID = $_SESSION['gollem']['params']['username'];
        }

        if (empty($credentials) &&
            !empty($_SESSION['gollem']['params']['password'])) {
            $credentials = array('password' => Secret::read(Secret::getKey('gollem'), $_SESSION['gollem']['params']['password']));
        }

        $login = ($login && ($this->getProvider() == 'gollem'));

        return parent::authenticate($userID, $credentials, $login);
    }

    /**
     * Find out if a set of login credentials are valid.
     *
     * @access private
     *
     * @param string $userID      The userID to check.
     * @param array $credentials  An array of login credentials.
     *
     * @return boolean  Whether or not the credentials are valid.
     */
    function _authenticate($userID, $credentials)
    {
        if (!(isset($_SESSION['gollem']) && is_array($_SESSION['gollem']))) {
            if (isset($GLOBALS['prefs'])) {
                $GLOBALS['prefs']->cleanup(true);
            }
            $this->_setAuthError(AUTH_REASON_SESSION);
            return false;
        }

        /* Unencrypt the password now. */
        $params = $_SESSION['gollem']['params'];
        if (!empty($params['password'])) {
            $params['password'] = Secret::read(Secret::getKey('gollem'), $params['password']);
        }

        $GLOBALS['vfs'] = &VFS::singleton($_SESSION['gollem']['driver'], $params);
        $valid = $GLOBALS['vfs']->checkCredentials();
        if (is_a($valid, 'PEAR_Error')) {
            $this->_setAuthError(AUTH_REASON_FAILED);
            return false;
        }

        return true;
    }

}
