<?php
/**
 * $Horde: gollem/manager.php,v 1.126 2005/01/03 14:34:54 jan Exp $
 *
 * Copyright 1999-2005 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999-2005 Max Kalika <max@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did notcan receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('GOLLEM_BASE', dirname(__FILE__));
require_once GOLLEM_BASE . '/lib/base.php';
require_once 'Horde/MIME/Part.php';
require_once 'Horde/Template.php';
require HORDE_BASE . '/config/mime_drivers.php';
require GOLLEM_BASE . '/config/mime_drivers.php';

$actionID = Util::getFormData('actionID');
$old_dir = Gollem::getDir();

/* Set directory. */
if (is_a($result = Gollem::changeDir(), 'PEAR_Error')) {
    $notification->push($result);
}

if (($actionID == 'login') && isset($new_lang)) {
    $language = $new_lang;
}

/* Check for 'sortby' and 'sortdir' changes. */
if (($sortby = Util::getFormData('sortby')) !== null) {
    $prefs->setValue('sortby', $sortby);
}
if (($sortdir = Util::getFormData('sortdir')) !== null) {
    $prefs->setValue('sortdir', $sortdir);
}

/* Run through the action handlers. */
switch ($actionID) {
case 'create_folder':
    if (Gollem::checkPermissions('backend', PERMS_EDIT)) {
        if ($new_folder = Util::getPost('new_folder')) {
            $result = Gollem::createFolder($old_dir, $new_folder);
            if (is_a($result, 'PEAR_Error')) {
                $notification->push($result->getMessage(), 'horde.error');
            } else {
                $notification->push(_("New folder created: ") . $new_folder, 'horde.success');
            }
        }
    } else {
        $notification->push(_("You do not have permission to create this folder."), 'horde.warning');
    }
    break;

case 'rename_items':
    if (Gollem::checkPermissions('backend', PERMS_EDIT)) {
        $new = explode('|', Util::getPost('new_names'));
        $old = explode('|', Util::getPost('old_names'));
        if (!empty($new) && !empty($old) && (count($new) == count($old))) {
            $iMax = count($new);
            for ($i = 0; $i < $iMax; $i++) {
                $result = Gollem::renameItem($old_dir, $old[$i], $old_dir, $new[$i]);
                if (is_a($result, 'PEAR_Error')) {
                    $notification->push($result->getMessage(), 'horde.error');
                } else {
                    $notification->push(sprintf(_("'%s' renamed to '%s'"), $old[$i], $new[$i]), 'horde.success');
                }
            }
        } else {
            $notification->push(_("Incorrect number of items."), 'horde.error');
        }
    } else {
        $notification->push(_("You do not have permission to rename items."), 'horde.warning');
    }
    break;

case 'chmod_modify':
case 'delete_items':
    if (Gollem::checkPermissions('backend', PERMS_DELETE)) {
        $items = Util::getPost('items');
        if (is_array($items) && count($items)) {
            $chmod = Util::getPost('chmod');
            foreach ($items as $item) {
                if (($actionID == 'chmod_modify') && $chmod) {
                    if (!is_a(Gollem::changePermissions(Gollem::getDir(), $item, $chmod), 'PEAR_Error')) {
                        $notification->push(_("Chmod done: ") . $item, 'horde.success');
                    } else {
                        $notification->push(sprintf(_("Cannot chmod %s"), $item), 'horde.error');
                    }
                } elseif ($actionID == 'delete_items') {
                    if (!is_a($result = Gollem::deleteFile($old_dir, $item), 'PEAR_Error')) {
                        $notification->push(_("File deleted: ") . $item, 'horde.success');
                    } elseif (!is_a($result = Gollem::deleteFolder($old_dir, $item), 'PEAR_Error')) {
                        $notification->push(_("Folder removed: ") . $item, 'horde.success');
                    } else {
                        $notification->push(sprintf(_("Cannot delete '%s': %s"), $item, $result->getMessage()), 'horde.error');
                    }
                }
            }
        }
    } else {
        $notification->push(_("You do not have permission to delete or change permissions on items."), 'horde.warning');
    }
    break;

case 'upload_file':
    if (Gollem::checkPermissions('backend', PERMS_EDIT)) {
       foreach (array('file_upload_1', 'file_upload_2', 'file_upload_3') as $val) {
           if (isset($_FILES[$val]) &&
               ($_FILES[$val]['error'] != 4)) {
                $res = Browser::wasFileUploaded($val);
                if (!is_a($res, 'PEAR_Error')) {
                    $res = Gollem::writeFile($old_dir, $_FILES[$val]['name'], $_FILES[$val]['tmp_name']);
                }
                if (is_a($res, 'PEAR_Error')) {
                    $notification->push($res, 'horde.error');
                } else {
                    $notification->push(sprintf(_("File received: %s"), $_FILES[$val]['name']), 'horde.success');
                }
            }
       }
    } else {
        $notification->push(_("You do not have permission to upload items."), 'horde.warning');
    }
    break;

case 'copy_items':
case 'cut_items':
    if (Gollem::checkPermissions('backend', PERMS_EDIT)) {
        $action = ($actionID == 'copy_items') ? 'copy' : 'cut';
        $items = Util::getPost('items');
        if (is_array($items) && count($items)) {
            $_SESSION['gollem'][$action]['selected'] = true;
            foreach ($items as $item) {
                $_SESSION['gollem'][$action]['names'][$item] = $item;
                $_SESSION['gollem'][$action]['path'] = $old_dir;
                if ($action == 'copy') {
                    $notification->push(sprintf(_("Item copied to clipboard: %s"), $item),'horde.success');
                } else {
                    $notification->push(sprintf(_("Item cut to clipboard: %s"), $item), 'horde.success');
                }
            }
        } else {
            if ($action == 'copy') {
                $notification->push(_("Cannot copy items onto clipboard."), 'horde.error');
            } else {
                $notification->push(_("Cannot cut items onto clipboard."), 'horde.error');
            }
        }
    } else {
        $notification->push(_("You do not have permission to cut/copy items."), 'horde.warning');
    }
    break;

case 'paste_items':
    if (Gollem::checkPermissions('backend', PERMS_EDIT)) {
        $action = (empty($_SESSION['gollem']['cut']['selected'])) ? 'copy' : 'cut';
        if (is_array($_SESSION['gollem'][$action]['names']) &&
            count($_SESSION['gollem'][$action]['names'])) {
            foreach ($_SESSION['gollem'][$action]['names'] as $item) {
                if ($action == 'cut') {
                    $res = Gollem::moveFile($_SESSION['gollem']['cut']['path'], $item, $old_dir);
                } else {
                    $res = Gollem::copyFile($_SESSION['gollem']['copy']['path'], $item, $old_dir);
                }
                if (is_a($res, 'PEAR_Error')) {
                    $notification->push(sprintf(_("Cannot paste '%s' (clipboard cleared): %s"), $item, $res->getMessage()), 'horde.error');
                } else {
                    $notification->push(sprintf(_("%s was successfully pasted."), $item, $old_dir), 'horde.success');
                }
            }
        }
        $_SESSION['gollem'][$action]['selected'] = false;
        $_SESSION['gollem'][$action]['names'] = array();
        $_SESSION['gollem'][$action]['path'] = '';
    } else {
        $notification->push(_("You do not have permission to paste items."), 'horde.warning');
    }
    break;
}

/* First loop through getting folder lists, setting the directory,
 * etc., to make sure we can catch any errors. */
$currdir = Gollem::getDir();
$info = array();

$info['list'] = Gollem::listFolder($currdir);
if (is_a($info['list'], 'PEAR_Error')) {
    /* If this is a user's home directory, try autocreating it. */
    if ($currdir == Gollem::getHome()) {
        Gollem::createFolder('', $currdir);
        $info['list'] = Gollem::listFolder($currdir);
    }

    /* If that didn't work, fall back to the parent or the home directory. */
    if (is_a($info['list'], 'PEAR_Error')) {
        $notification->push(sprintf(_("Permission denied to %s: %s"), $currdir, $info['list']->getMessage()), 'horde.error');

        $loc = strrpos($currdir, '/');
        if ($loc !== false) {
            Gollem::setDir(substr($currdir, 0, $loc));
        } else {
            Gollem::setDir(Gollem::getHome());
        }
        $currdir = Gollem::getDir();
        $info['list'] = Gollem::listFolder($currdir);
    }
}

$info['title'] = htmlspecialchars($_SESSION['gollem']['label']);

require GOLLEM_TEMPLATES . '/common-header.inc';
require GOLLEM_TEMPLATES . '/menu.inc';
require_once $registry->get('templates', 'horde') . '/contents/open_view_win.js';

/* Image links. */
$download_img = Horde::img('download.png', _("Download"), null, $registry->getImageDir('horde'));
$folder_img = Horde::img('manager/folder.png', _("folder"));
$symlink_img = Horde::img('manager/symlink.png', _("symlink"));

/* Commonly used URLs. */
$manager_url = Horde::applicationUrl('manager.php');
$view_url = Horde::applicationUrl('view.php');

/* Get copy/cut paths. */
$copy_path = isset($_SESSION['gollem']['copy']['path']) ? $_SESSION['gollem']['copy']['path'] : null;
$cut_path = isset($_SESSION['gollem']['cut']['path']) ? $_SESSION['gollem']['cut']['path'] : null;

/* Now actually display everything, after we've notified the user of
 * any errors. */
$backkey = $_SESSION['gollem']['backend_key'];
$list = $info['list'];
$title = $info['title'];

require GOLLEM_TEMPLATES . '/manager/header.inc';

/* Read the columns to display from the preferences. */
$sources = Gollem::string2Columns($prefs->getValue('columns'));
$columns = isset($sources[$backkey]) ? $sources[$backkey] : $_SESSION['gollem']['attributes'];

/* Set up column listing for cloop. */
$columns_cloop = array();
foreach ($columns as $val) {
    $columns_cloop[] = array('case' => $val);
}

$icon_cache = array();

if (is_array($list) &&
    count($list) &&
    Gollem::checkPermissions('backend', PERMS_READ)) {
    require GOLLEM_TEMPLATES . '/manager/file_headers.inc';

    $entry = array();
    $total_files = $total_folders = $total_size = 0;

    foreach ($list as $key => $val) {
        $item = array();

        $item['date'] = strftime($conf['manager']['date_format'], $val['date']);
        $item['dl'] = false;
        $item['group'] = $val['group'];
        $item['name'] = htmlspecialchars($val['name']);
        $item['on_clipboard'] = false;
        $item['owner'] = $val['owner'];
        $item['perms'] = $val['perms'];
        $item['size'] = ($val['type'] == '**dir') ? '-' : number_format($val['size'], 0, '.', ',');
        $item['type'] = $val['type'];

        $name = str_replace(' ', '&nbsp;', $item['name']);

        /* Is this file on the clipboard? */
        if (($copy_path == $currdir) || ($cut_path == $currdir)) {
            if ((isset($_SESSION['gollem']['copy']['names'][$val['name']]) ||
                 isset($_SESSION['gollem']['cut']['names'][$val['name']])) ) {
                $item['on_clipboard'] = true;
            }
        }

        /* Determine graphic to use. */
        if (!empty($val['link'])) {
            $item['graphic'] = $symlink_img;
        } elseif ($val['type'] == '**dir') {
            $item['graphic'] = $folder_img;
        } else {
            if (empty($icon_cache[$val['type']])) {
                require_once 'Horde/MIME/Magic.php';
                require_once 'Horde/MIME/Viewer.php';
                $icon_cache[$val['type']] = Horde::img(MIME_Viewer::getIcon(MIME_Magic::extToMIME($val['type'])), '', '', '');
            }
            $item['graphic'] = $icon_cache[$val['type']];
        }

        /* Create proper link. */
        switch ($val['type']) {
        case '**dir':
            $url = Util::addParameter($manager_url, 'dir', Gollem::subdirectory($currdir, $val['name']));
            $item['link'] = Horde::link($url, $val['name']) . '<b>' . $name . '</b></a>';
            break;

        case '**broken':
            $item['link'] = $name;
            break;

        case '**sym':
            if ($val['linktype'] === '**dir') {
                if (substr($val['link'], 0, 1) == '/') {
                    $parts = explode('/', $val['link']);
                    $name = array_pop($parts);
                    $dir = implode('/', $parts);
                } else {
                    $name = $val['link'];
                    $dir = $currdir;
                }

                $url = Util::addParameter($manager_url, 'dir', Gollem::subdirectory($dir, $name));
                $item['link'] = $item['name'] . ' -> <b>' . Horde::link($url, $val['link']) . $val['link'] . '</a></b>';
            } else {
                $item['link'] = $item['name'] . ' -> ' . $val['link'];
            }
            break;

        default:
            require_once 'Horde/MIME/Magic.php';
            $mime_part = &new MIME_Part(MIME_Magic::extToMIME($val['type']), '');
            // We can always download files.
            $item['dl'] = Horde::link(Horde::downloadUrl($val['name'], array('actionID' => 'download_file', 'file' => $val['name'])), $val['name']) . $download_img . '</a>';

            // Try a view link.
            $url = Util::addParameter($view_url, array('actionID' => 'view_file', 'type' => $val['type'], 'file' => $val['name']));
            $item['link'] = Horde::link('', $val['name'], '', '', "view('$url', '" . $val['name'] . "'); return false;") . $name . '</a>';
            break;
        }

        if ($val['type'] != '**dir') {
            $total_size += $val['size'];
            $total_files++;
        } else {
            $total_folders++;
        }

        $entry[] = $item;
    }

    /* Set up the template tags. */
    $template = &new Horde_Template();
    $template->set('entry', $entry, true);
    $template->setCloop('columns', $columns_cloop, $columns);
    echo $template->fetch(GOLLEM_TEMPLATES . '/manager/manager.html');

    require GOLLEM_TEMPLATES . '/manager/file_footers.inc';
} else {
    require GOLLEM_TEMPLATES . '/manager/empty_dir.inc';
}

require GOLLEM_TEMPLATES . '/manager/footer.inc';
require $registry->get('templates', 'horde') . '/common-footer.inc';
